class VendorInfo:
    def __init__(self, vendor):
        self.__name = vendor.get("name")
        self.__modelName = vendor.get("modelName")
        self.__serialNo = vendor.get("serialNo")

    @property
    def name(self):
        return self.__name

    @property
    def modelName(self):
        return self.__modelName

    @property
    def serialNo(self):
        return self.__serialNo



import json
import urllib

from .models.TestObjectInfo import TestObjectInfo
from .models.CustomProperties import CustomProperties

#Services commands
from testwizard.commands_services import SendNotificationCommand
#Video commands
from testwizard.commands_video import SetTextOnScreenDisplayCommand
from testwizard.commands_video import SetAttributeOnScreenDisplayCommand
from testwizard.commands_video import ClearOnScreenDisplayCommand

class TestObjectBase():
    def __init__(self, session, name, category):
        if session is None:
            raise Exception("Session is required")
        if name is None:
            raise Exception("Name is required")
        if category is None:
            raise Exception("Category is required")

        if session.robot is None:
            raise Exception("Robot is undefined for session")

        resources = session.metadata["resources"]
        for resource1 in resources:
            if resource1["name"] == name:
                resource = resource1
                break
            else:
                resource = None
        if resource is None:
            raise Exception("No resource found with name " + name)
        
        testobject = session.robot.getTestObject(resource["id"])
        self.info = TestObjectInfo(testobject)
        self.customProperties = CustomProperties(resource.get("customProperties", {}))

        self.session = session
        self.name = name

        self.__isDisposed = False
        

    def executeCommand(self, commandName, requestObj, errorMessagePrefix):
        return self.session.robot.executeCommand(self.session.testRunId, self.name, commandName, requestObj, errorMessagePrefix)

    def dispose(self):
        self.__isDisposed = True

    def throwIfDisposed(self):
        if self.__isDisposed is True:
            print("Cannot access a disposed object")
            raise Exception("Cannot access a disposed object.")

    def sendNotification(self, errorId, description, priority):
        self.throwIfDisposed()
        return SendNotificationCommand(self).execute(errorId, description, priority)
        
    def setAttributeOnScreenDisplay(self, attributeType, osdArea = None,  textColor = None, backgroundColor = None, duration = None):
        self.throwIfDisposed()
        return SetAttributeOnScreenDisplayCommand(self).execute(attributeType, osdArea,  textColor, backgroundColor, duration)
        
    def setTextOnScreenDisplay(self, osdText, osdArea = None,  textColor = None, backgroundColor = None, duration = None):
        self.throwIfDisposed()
        return SetTextOnScreenDisplayCommand(self).execute(osdText, osdArea,  textColor, backgroundColor, duration)
        
    def clearOnScreenDisplay(self, osdArea = None):
        self.throwIfDisposed()
        return ClearOnScreenDisplayCommand(self).execute(osdArea)


from .VendorInfo import VendorInfo

class DeviceInfo:
    def __init__(self, device):
        self.__softwareVersion = device.get("softwareVersion")
        self.__hardwareVersion = device.get("hardwareVersion")
        self.__serialNo = device.get("serialNo")
        self.__vendor = VendorInfo(device.get("vendor"))
        self.__description = device.get("description")

    @property
    def softwareVersion(self):
        return self.__softwareVersion

    @property
    def hardwareVersion(self):
        return self.__hardwareVersion

    @property
    def serialNo(self):
        return self.__serialNo

    @property
    def vendor(self):
        return self.__vendor

    @property
    def description(self):
        return self.__description



from .VideoInfo import VideoInfo
from .AudioInfo import AudioInfo
from .RemoteControlInfo import RemoteControlInfo
from .DeviceInfo import DeviceInfo
from .MobileInfo import MobileInfo

class TestObjectInfo:
    def __init__(self, testObject):
        self.__id = testObject.get("id")
        self.__name = testObject.get("name")
        self.__category = testObject.get("category")
        self.__isEnabled = testObject.get("isEnabled")
        self.__isLocked = testObject.get("isLocked")
        self.__video = VideoInfo(testObject.get("video"))
        self.__audio = AudioInfo(testObject.get("audio"))
        self.__remoteControl = RemoteControlInfo(testObject.get("remoteControl"))
        self.__device = DeviceInfo(testObject.get("device"))
        self.__mobile = MobileInfo(testObject.get("mobile") or {})

    @property
    def id(self):
        return self.__id

    @property
    def name(self):
        return self.__name

    @property
    def category(self):
        return self.__category

    @property
    def isEnabled(self):
        return self.__isEnabled

    @property
    def isLocked(self):
        return self.__isLocked

    @property
    def video(self):
        return self.__video

    @property
    def audio(self):
        return self.__audio

    @property
    def remoteControl(self):
        return self.__remoteControl

    @property
    def device(self):
        return self.__device

    @property
    def mobile(self):
        return self.__mobile


class CustomProperties():
    def __init__(self, jsonObj):
        self.__dict = jsonObj

        for name, value in jsonObj.items():
            setattr(self, name, value)
    
    def __getitem__(self, name):
        return self.__dict.get(name)

    def __iter__(self):
        return iter(self.__dict)

    def keys(self):
        return self.__dict.keys()

    def values(self):
        return self.__dict.values()

    def itervalues(self):
        return self.__dict.itervalues()

    def get(self, name, default=None):
        return self.__dict.get(name, default)


class RemoteControlInfo:
    def __init__(self, remoteControl):
        self.__isSupported = remoteControl.get("isSupported")
        self.__name = remoteControl.get("name")

    @property
    def isSupported(self):
        return self.__isSupported

    @property
    def name(self):
        return self.__name
